from django.contrib import admin, messages
from django.contrib.auth.mixins import UserPassesTestMixin
from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.http.response import HttpResponse
from django.urls import reverse_lazy
from django.views.generic import TemplateView

from adminutilities.utils import execute_module_function


class CustomFunctionFormView(UserPassesTestMixin, TemplateView):
    """
    A view for executing custom functions via the admin interface.

    This view requires the user to be a superuser in order to execute functions.
    If the user is not a superuser, they will be redirected to the login page or receive a permission denied message.

    Attributes:
        template_name (str): The name of the template to be rendered.
        success_url (str): The URL to redirect to after executing the functions.
        permission_denied_message (str): The message to display when the user does not have permission to execute functions.
        raise_exception (bool): Whether to raise a PermissionDenied exception when the user does not have permission.
        login_url (str): The URL to redirect to if the user is not authenticated.

    Methods:
        handle_no_permission: Handles the case when the user does not have permission to execute functions.
        test_func: Checks if the user is a superuser.
        execute_function: Executes a custom function given its path.
        post: Handles the POST request and executes the selected functions.
        get_context_data: Adds additional context data to be passed to the template.
    """

    template_name = "adminutilities/admin/customfunction_form.html"
    success_url = reverse_lazy("customcunction_admin")
    permission_denied_message = "Only superuser can execute via admin."
    raise_exception = False
    login_url = None

    def handle_no_permission(self):
        """
        Handles the case when the user does not have permission to execute functions.

        If `raise_exception` is True or the user is authenticated, a PermissionDenied exception is raised.
        Otherwise, a simple HTML response with a login message is returned.
        """
        if self.raise_exception or self.request.user.is_authenticated:
            raise PermissionDenied(self.get_permission_denied_message())
        return HttpResponse("<h1>please login</h1>")

    def test_func(self):
        """
        Checks if the user is a superuser.

        Returns:
            bool: True if the user is a superuser, False otherwise.
        """
        return self.request.user.is_superuser

    def execute_function(self, function_path: str):
        """
        Executes a custom function given its path.

        Args:
            function_path (str): The path to the function to be executed.
        """
        execute_module_function(function_path)

    def post(self, request, *args, **kwargs):
        """
        Handles the POST request and executes the selected functions.

        Args:
            request (HttpRequest): The HTTP request object.

        Returns:
            HttpResponseRedirect: The response object for redirecting to the success URL.
        """
        function_paths = request.POST.getlist("function_path", None)

        if function_paths:
            try:
                for function_path in function_paths:
                    self.execute_function(function_path)
                    messages.success(self.request, f"Function {function_path} successfully")

            except Exception as err:
                messages.error(
                    self.request,
                    f"Couldn't execute selected operation, something went wrong. Received error: {err}",
                )
        else:
            messages.warning(self.request, f"Has no selected operation to do!")

        return HttpResponseRedirect(self.success_url)

    def get_context_data(self, **kwargs):
        """
        Adds additional context data to be passed to the template.

        Returns:
            dict: The context data.
        """
        context = super().get_context_data(**kwargs)
        assert hasattr(self.request, "admin_tool_functions"), "Please add 'adminutilities.middleware.GetAllAdminToolFunctionsMiddleware' " "to MIDDLEWARE in settings.py"
        context["admin_tool_functions"] = dict(self.request.admin_tool_functions)
        context["title"] = "Execute custom function"
        context["site_header"] = admin.site.site_header
        context["has_permission"] = True
        return context



import importlib
import os
from collections import defaultdict

from django.apps import apps


def get_register_admin_tool_functions():
    """
    Get a dictionary of registered admin tool functions.

    Returns:
        A dictionary where the keys are the app names and the values are lists of tuples
        containing the tool function name and path.
    """
    admin_tool_functions = defaultdict(list)
    installed_apps = apps.get_app_configs()

    for app in installed_apps:
        admin_tools_path = os.path.join(app.path, "admin_tools.py")

        if os.path.exists(admin_tools_path):
            admin_tools_module = importlib.import_module(app.name + ".admin_tools")
            functions = [name for name, obj in vars(admin_tools_module).items() if callable(obj) and hasattr(obj, "is_admin_tools") and getattr(obj, "is_admin_tools")]

            for function_name in functions:
                function_path = f"{app.name}.{function_name}"
                admin_tool_functions[app.name].append((function_name, function_path))

    return admin_tool_functions


def get_register_admin_tool_function_paths() -> list:
    """
    Get a list of registered admin tool function paths.

    Returns:
        A list of function paths in the format 'app_name.function_name'.
    """
    register_admin_tool_functions = get_register_admin_tool_functions()

    function_paths = []

    for _, function_infos in register_admin_tool_functions.items():
        for function_info in function_infos:
            function_paths.append(function_info[1])

    return function_paths


def execute_module_function(function_path: str):
    """
    Execute a function in the module's admin_tools.

    Args:
        function_path: The path of the function in the format 'app_name.function_name'.

    Raises:
        ImportError: If the function_path is not a valid module path.
        ImportError: If the module does not define the specified function.
    """
    try:
        app_name, function_name = function_path.split(".")
        module_path = app_name + ".admin_tools"
    except ValueError as err:
        raise ImportError("%s doesn't look like a module path" % function_path) from err

    admin_tools_module = importlib.import_module(module_path)

    try:
        getattr(admin_tools_module, function_name)()
    except AttributeError as err:
        raise ImportError('Module "%s" does not define a "%s" attribute/function' % (module_path, function_name)) from err



from functools import wraps


def admin_tool(func):
    """
    Decorator function to mark a function as an admin tool.

    Args:
        func: The function to be marked as an admin tool.

    Returns:
        The wrapped function.
    """
    setattr(func, "is_admin_tools", True)

    @wraps(func)
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)

    return wrapper


from django.utils.deprecation import MiddlewareMixin
from django.utils.functional import SimpleLazyObject

from adminutilities.utils import get_register_admin_tool_functions


def get_admin_tool_functions(request):
    """
    Retrieves the admin tool functions for the given request.

    Args:
        request (HttpRequest): The request object.

    Returns:
        list: The list of admin tool functions.
    """
    if not hasattr(request, "_cached_tool_functions"):
        request._cached_tool_functions = get_register_admin_tool_functions()
    return request._cached_tool_functions


class GetAllAdminToolFunctionsMiddleware(MiddlewareMixin):
    def process_request(self, request):
        """
        Adds the admin tool functions to the request object.

        Args:
            request (HttpRequest): The request object.
        """
        request.admin_tool_functions = SimpleLazyObject(lambda: get_admin_tool_functions(request))



from django.core.management.base import BaseCommand

from adminutilities.utils import get_register_admin_tool_function_paths, execute_module_function


class Command(BaseCommand):
    help = "Execute adminutilities custom function"

    def add_arguments(self, parser):
        parser.add_argument("function_path", nargs="+", type=str, help=f"Current optional: {', '.join(get_register_admin_tool_function_paths())}")

    def handle(self, *args, **options):
        function_paths = options["function_path"]

        for function_path in function_paths:
            try:
                execute_module_function(function_path)
                self.stdout.write(self.style.SUCCESS(f"Function {function_path} successfully"))
            except Exception as err:
                self.stderr.write(self.style.ERROR(err))



#!/usr/bin/env python

"""The setup script."""
import os
import re
from setuptools import setup, find_packages


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)

latest_version = get_version('adminutilities')

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [ ]

test_requirements = [ ]

setup(
    author="tangmeijian",
    author_email='tang1996mei@gmail.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    description="Allow calling registered custom functions (system tools, script functions, etc.)  via admin UI or manage.py command.",
    install_requires=requirements,
    license="BSD license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords=['django', 'utilities', 'admin'],
    name='django-adminutilities',
    packages=find_packages(include=['adminutilities', 'adminutilities.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/Skylor-Tang/django-adminutilities',
    version=latest_version,
    zip_safe=False,
)
